@@ -13,7 +13,8 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
 import collections
 
@@ -109,7 +110,7 @@ class DiffMap(gtk.DrawingArea):
         for c, y0, y1 in self._difffunc():
             tagged_diffs[c].append((y0, y1))
 
-        for tag, diffs in tagged_diffs.iteritems():
+        for tag, diffs in tagged_diffs.items():
             context.set_source_color(self.fill_colors[tag])
             for y0, y1 in diffs:
                 y0, y1 = round(y0 * height) - 0.5, round(y1 * height) - 0.5
