@@ -13,13 +13,14 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
 import difflib
 
 import gobject
 
-from matchers import DiffChunk, MyersSequenceMatcher
+from .matchers import DiffChunk, MyersSequenceMatcher
 
 
 opcode_reverse = {
@@ -41,9 +42,9 @@ def consume_blank_lines(chunk, texts, pane1, pane2):
         return None
 
     def _find_blank_lines(txt, lo, hi):
-        while not txt[lo] and lo < hi:
+        while lo < hi and not txt[lo]:
             lo += 1
-        while not txt[hi - 1] and lo < hi:
+        while lo < hi and not txt[hi - 1]:
             hi -= 1
         return lo, hi
 
@@ -54,9 +55,9 @@ def consume_blank_lines(chunk, texts, pane1, pane2):
     if c1 == c2 and c3 == c4:
         return None
     if c1 == c2 and tag == "replace":
-        c0 = "insert"
+        tag = "insert"
     elif c3 == c4 and tag == "replace":
-        c0 = "delete"
+        tag = "delete"
     return DiffChunk._make((tag, c1, c2, c3, c4))
 
 
@@ -336,12 +337,12 @@ class Differ(gobject.GObject):
             merge_cache = self._merge_cache
 
         if fromindex == 1:
-            seq = toindex/2
+            seq = toindex // 2
             for c in merge_cache:
                 if c[seq]:
                     yield c[seq]
         else:
-            seq = fromindex/2
+            seq = fromindex // 2
             for c in merge_cache:
                 if c[seq]:
                     yield reverse_chunk(c[seq])
@@ -357,7 +358,7 @@ class Differ(gobject.GObject):
         else:
             merge_cache = self._merge_cache
         if textindex in (0,2):
-            seq = textindex/2
+            seq = textindex // 2
             for cs in merge_cache:
                 if cs[seq]:
                     yield reverse_chunk(cs[seq])
@@ -455,7 +456,7 @@ class Differ(gobject.GObject):
         for i in range(self.num_sequences - 1):
             matcher = self._matcher(None, sequences[1], sequences[i*2])
             work = matcher.initialise()
-            while work.next() is None:
+            while next(work) is None:
                 yield None
             self.diffs[i] = matcher.get_difference_opcodes()
         self._initialised = True
