@@ -1,5 +1,5 @@
 ### Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
-### Copyright (C) 2009-2011 Kai Willadsen <kai.willadsen@gmail.com>
+### Copyright (C) 2009-2012 Kai Willadsen <kai.willadsen@gmail.com>
 
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -13,30 +13,35 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
 import collections
 import copy
+import datetime
 import errno
-import paths
-from ui import gnomeglade
-import gtk
-import gtk.keysyms
-import misc
 import os
-from gettext import gettext as _
-from gettext import ngettext
-import shutil
-import melddoc
-import tree
 import re
+import shutil
 import stat
 import time
 
-import ui.emblemcellrenderer
+import gtk
+import gtk.keysyms
+
+from . import melddoc
+from . import tree
+from . import misc
+from . import paths
+from . import recent
+from .ui import gnomeglade
+from .ui import emblemcellrenderer
 
 from collections import namedtuple
-from meldapp import app
+from decimal import Decimal
+from gettext import gettext as _
+from gettext import ngettext
+from .meldapp import app
 
 gdk = gtk.gdk
 
@@ -54,12 +59,29 @@ class StatItem(namedtuple('StatItem', 'mode size time')):
         return StatItem(stat.S_IFMT(stat_result.st_mode),
                         stat_result.st_size, stat_result.st_mtime)
 
+    def shallow_equal(self, other, prefs):
+        if self.size != other.size:
+            return False
+
+        # Shortcut to avoid expensive Decimal calculations. 2 seconds is our
+        # current accuracy threshold (for VFAT), so should be safe for now.
+        if abs(self.time - other.time) > 2:
+            return False
+
+        dectime1 = Decimal(str(self.time)).scaleb(Decimal(9)).quantize(1)
+        dectime2 = Decimal(str(other.time)).scaleb(Decimal(9)).quantize(1)
+        mtime1 = dectime1 // prefs.dirdiff_time_resolution_ns
+        mtime2 = dectime2 // prefs.dirdiff_time_resolution_ns
+
+        return mtime1 == mtime2
+
 
 CacheResult = namedtuple('CacheResult', 'stats result')
 
 
 _cache = {}
-Same, SameFiltered, DodgySame, DodgyDifferent, Different, FileError = range(6)
+Same, SameFiltered, DodgySame, DodgyDifferent, Different, FileError = \
+    list(range(6))
 # TODO: Get the block size from os.stat
 CHUNK_SIZE = 4096
 
@@ -68,7 +90,7 @@ def all_same(lst):
     return not lst or lst.count(lst[0]) == len(lst)
 
 
-def _files_same(files, regexes):
+def _files_same(files, regexes, prefs):
     """Determine whether a list of files are the same.
 
     Possible results are:
@@ -95,6 +117,13 @@ def _files_same(files, regexes):
     if not all([stat.S_ISREG(s.mode) for s in stats]):
         return Different
 
+    # Compare files superficially if the options tells us to
+    if prefs.dirdiff_shallow_comparison:
+        if all(s.shallow_equal(stats[0], prefs) for s in stats[1:]):
+            return DodgySame
+        else:
+            return Different
+
     # If there are no text filters, unequal sizes imply a difference
     if not regexes and not all_same([s.size for s in stats]):
         return Different
@@ -156,16 +185,13 @@ def _files_same(files, regexes):
     return result
 
 
-COL_EMBLEM, COL_END = tree.COL_END, tree.COL_END + 1
+COL_EMBLEM, COL_SIZE, COL_TIME, COL_PERMS, COL_END = \
+        range(tree.COL_END, tree.COL_END + 5)
+
 
-################################################################################
-#
-# DirDiffTreeStore
-#
-################################################################################
 class DirDiffTreeStore(tree.DiffTreeStore):
     def __init__(self, ntree):
-        tree.DiffTreeStore.__init__(self, ntree, [str])
+        tree.DiffTreeStore.__init__(self, ntree, [str, str, str, str])
 
 
 class CanonicalListing(object):
@@ -191,7 +217,7 @@ class CanonicalListing(object):
     def get(self):
         first = lambda seq: next(s for s in seq if s)
         filled = lambda seq: tuple([s or first(seq) for s in seq])
-        return sorted([filled(v) for v in self.items.itervalues()])
+        return sorted(filled(v) for v in self.items.values())
 
 
 ################################################################################
@@ -266,10 +292,12 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.widget.connect("style-set", self.on_style_set)
         self.widget.ensure_style()
 
+        self.custom_labels = []
         self.set_num_panes(num_panes)
 
         self.widget.connect("style-set", self.model.on_style_set)
 
+        self.do_to_others_lock = False
         self.focus_in_events = []
         self.focus_out_events = []
         for treeview in self.treeview:
@@ -279,7 +307,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.focus_out_events.append(handler_id)
         self.current_path, self.prev_path, self.next_path = None, None, None
         self.on_treeview_focus_out_event(None, None)
-        self.treeview_focussed = None
+        self.focus_pane = None
 
         lastchanged_label = gtk.Label()
         lastchanged_label.set_size_request(100, -1)
@@ -289,15 +317,17 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         permissions_label.show()
         self.status_info_labels = [lastchanged_label, permissions_label]
 
+        # One column-dict for each treeview, for changing visibility and order
+        self.columns_dict = [{}, {}, {}]
         for i in range(3):
-            self.treeview[i].get_selection().set_mode(gtk.SELECTION_MULTIPLE)
-            column = gtk.TreeViewColumn()
+            col_index = self.model.column_index
+            # Create icon and filename CellRenderer
+            column = gtk.TreeViewColumn(_("Name"))
             rentext = gtk.CellRendererText()
-            renicon = ui.emblemcellrenderer.EmblemCellRenderer()
+            renicon = emblemcellrenderer.EmblemCellRenderer()
             column.pack_start(renicon, expand=0)
             column.pack_start(rentext, expand=1)
-            col_index = self.model.column_index
-            column.set_attributes(rentext, text=col_index(tree.COL_TEXT,i),
+            column.set_attributes(rentext, text=col_index(tree.COL_TEXT, i),
                                   foreground_gdk=col_index(tree.COL_FG, i),
                                   style=col_index(tree.COL_STYLE, i),
                                   weight=col_index(tree.COL_WEIGHT, i),
@@ -307,6 +337,32 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                                   emblem_name=col_index(COL_EMBLEM, i),
                                   icon_tint=col_index(tree.COL_TINT, i))
             self.treeview[i].append_column(column)
+            self.columns_dict[i]["name"] = column
+            # Create file size CellRenderer
+            column = gtk.TreeViewColumn(_("Size"))
+            rentext = gtk.CellRendererText()
+            column.pack_start(rentext, expand=1)
+            column.set_attributes(rentext, markup=col_index(COL_SIZE, i))
+            self.treeview[i].append_column(column)
+            self.columns_dict[i]["size"] = column
+            # Create date-time CellRenderer
+            column = gtk.TreeViewColumn(_("Modification time"))
+            rentext = gtk.CellRendererText()
+            column.pack_start(rentext, expand=1)
+            column.set_attributes(rentext, markup=col_index(COL_TIME, i))
+            self.treeview[i].append_column(column)
+            self.columns_dict[i]["modification time"] = column
+            # Create permissions CellRenderer
+            column = gtk.TreeViewColumn(_("Permissions"))
+            rentext = gtk.CellRendererText()
+            column.pack_start(rentext, expand=0)
+            column.set_attributes(rentext, markup=col_index(COL_PERMS, i))
+            self.treeview[i].append_column(column)
+            self.columns_dict[i]["permissions"] = column
+        self.update_treeview_columns(self.prefs.dirdiff_columns)
+
+        for i in range(3):
+            self.treeview[i].get_selection().set_mode(gtk.SELECTION_MULTIPLE)
             self.scrolledwindow[i].get_vadjustment().connect("value-changed", self._sync_vscroll )
             self.scrolledwindow[i].get_hadjustment().connect("value-changed", self._sync_hscroll )
         self.linediffs = [[], []]
@@ -343,6 +399,29 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         for diffmap in self.diffmap:
             diffmap.queue_draw()
 
+    def on_preference_changed(self, key, value):
+        if key == "dirdiff_columns":
+            self.update_treeview_columns(value)
+        elif key == "dirdiff_shallow_comparison":
+            self.refresh()
+        elif key == "dirdiff_time_resolution_ns":
+            self.refresh()
+
+    def update_treeview_columns(self, columns):
+        """Update the visibility and order of columns"""
+        for i in range(3):
+            extra_cols = False
+            last_column = self.treeview[i].get_column(0)
+            for line in columns:
+                column_name, visible = line.rsplit(" ", 1)
+                visible = bool(int(visible))
+                extra_cols = extra_cols or visible
+                current_column = self.columns_dict[i][column_name]
+                current_column.set_visible(visible)
+                self.treeview[i].move_column_after(current_column, last_column)
+                last_column = current_column
+            self.treeview[i].set_headers_visible(extra_cols)
+
     def on_custom_filter_menu_toggled(self, item):
         if item.get_active():
             self.custom_popup.connect("deactivate",
@@ -375,10 +454,6 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self._create_filter_menu_button(ui)
         self.ui_manager = ui
 
-        if self.treeview_focussed:
-            self.scheduler.add_task(self.treeview_focussed.grab_focus)
-            self.scheduler.add_task(self.on_treeview_cursor_changed)
-
     def on_container_switch_out_event(self, ui):
         self._cleanup_filter_menu_button(ui)
         melddoc.MeldDoc.on_container_switch_out_event(self, ui)
@@ -432,29 +507,31 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         return active_filters_changed
 
     def _do_to_others(self, master, objects, methodname, args):
-        if not hasattr(self, "do_to_others_lock"):
-            self.do_to_others_lock = 1
-            try:
-                for o in filter(lambda x:x!=master, objects[:self.num_panes]):
-                    method = getattr(o,methodname)
-                    method(*args)
-            finally:
-                delattr(self, "do_to_others_lock")
+        if self.do_to_others_lock:
+            return
+
+        self.do_to_others_lock = True
+        try:
+            others = [o for o in objects[:self.num_panes] if o != master]
+            for o in others:
+                method = getattr(o, methodname)
+                method(*args)
+        finally:
+            self.do_to_others_lock = False
 
     def _sync_vscroll(self, adjustment):
-        adjs = map(lambda x: x.get_vadjustment(), self.scrolledwindow)
-        self._do_to_others( adjustment, adjs, "set_value", (adjustment.value,) )
+        adjs = [sw.get_vadjustment() for sw in self.scrolledwindow]
+        self._do_to_others(adjustment, adjs, "set_value", (adjustment.value, ))
 
     def _sync_hscroll(self, adjustment):
-        adjs = map(lambda x: x.get_hadjustment(), self.scrolledwindow)
-        self._do_to_others( adjustment, adjs, "set_value", (adjustment.value,) )
+        adjs = [sw.get_hadjustment() for sw in self.scrolledwindow]
+        self._do_to_others(adjustment, adjs, "set_value", (adjustment.value, ))
 
     def _get_focused_pane(self):
-        focus = [ t.is_focus() for t in self.treeview ]
-        try:
-            return focus.index(1)
-        except ValueError:
-            return None
+        for i, treeview in enumerate(self.treeview):
+            if treeview.is_focus():
+                return i
+        return None
 
     def file_deleted(self, path, pane):
         # is file still extant in other pane?
@@ -494,6 +571,14 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.recursively_update( (0,) )
         self._update_diffmaps()
 
+    def get_comparison(self):
+        root = self.model.get_iter_root()
+        if root:
+            folders = self.model.value_paths(root)
+        else:
+            folders = []
+        return recent.TYPE_FOLDER, folders
+
     def recursively_update( self, path ):
         """Recursively update from tree path 'path'.
         """
@@ -503,7 +588,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.model.remove(child)
             child = self.model.iter_children( it )
         self._update_item_state(it)
-        self.scheduler.add_task( self._search_recursively_iter( path ).next )
+        self.scheduler.add_task(self._search_recursively_iter(path))
 
     def _search_recursively_iter(self, rootpath):
         self.actiongroup.get_action("Hide").set_sensitive(False)
@@ -627,6 +712,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.treeview[0].expand_to_path(path)
         yield _("[%s] Done") % self.label_text
         self.actiongroup.get_action("Hide").set_sensitive(True)
+        self.scheduler.add_task(self.on_treeview_cursor_changed)
 
     def _show_tree_wide_errors(self, invalid_filenames, shadowed_entries):
         header = _("Multiple errors occurred while scanning this folder")
@@ -679,24 +765,6 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         msgarea.show_all()
         return msgarea
 
-    def launch_comparison(self, it, pane, force=1):
-        """Launch comparison at 'it'. 
-           If it is a file we launch a diff.
-           If it is a folder we recursively open diffs for each non equal file.
-        """
-        paths = filter(os.path.exists, self.model.value_paths(it))
-        self.emit("create-diff", paths)
-
-    def launch_comparisons_on_selected(self):
-        """Launch comparisons on all selected elements.
-        """
-        pane = self._get_focused_pane()
-        if pane is not None:
-            selected = self._get_selected_paths(pane)
-            get_iter = self.model.get_iter
-            for s in selected:
-                self.launch_comparison( get_iter(s), pane )
-
     def copy_selected(self, direction):
         assert direction in (-1,1)
         src_pane = self._get_focused_pane()
@@ -802,8 +870,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
 
         if self.current_path == path:
             self.current_path = refocus_deleted_path(model, path)
-            if self.current_path and self.treeview_focussed:
-                self.treeview_focussed.set_cursor(self.current_path)
+            if self.current_path and self.focus_pane:
+                self.focus_pane.set_cursor(self.current_path)
 
     def on_treeview_cursor_changed(self, *args):
         pane = self._get_focused_pane()
@@ -813,6 +881,12 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         cursor_path, cursor_col = self.treeview[pane].get_cursor()
         if not cursor_path:
             self.emit("next-diff-changed", False, False)
+            self.current_path = cursor_path
+            return
+
+        # If invoked directly rather than through a callback, we always check
+        if not args:
+            skip = False
         else:
             try:
                 old_cursor = self.model.get_iter(self.current_path)
@@ -820,17 +894,26 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 # An invalid path gives ValueError; None gives a TypeError
                 skip = False
             else:
+                # We can skip recalculation if the new cursor is between
+                # the previous/next bounds, and we weren't on a changed row
                 state = self.model.get_state(old_cursor, 0)
-                # We can skip recalculation if the new cursor is between the
-                # previous/next bounds, and we weren't on a changed row
-                skip = state in (tree.STATE_NORMAL, tree.STATE_EMPTY) and \
-                       self.prev_path < cursor_path < self.next_path
+                if state not in (tree.STATE_NORMAL, tree.STATE_EMPTY):
+                    skip = False
+                else:
+                    if self.prev_path is None and self.next_path is None:
+                        skip = True
+                    elif self.prev_path is None:
+                        skip = cursor_path < self.next_path
+                    elif self.next_path is None:
+                        skip = self.prev_path < cursor_path
+                    else:
+                        skip = self.prev_path < cursor_path < self.next_path
 
-            if not skip:
-                prev, next = self.model._find_next_prev_diff(cursor_path)
-                self.prev_path, self.next_path = prev, next
-                have_next_diffs = (prev is not None, next is not None)
-                self.emit("next-diff-changed", *have_next_diffs)
+        if not skip:
+            prev, next = self.model._find_next_prev_diff(cursor_path)
+            self.prev_path, self.next_path = prev, next
+            have_next_diffs = (prev is not None, next is not None)
+            self.emit("next-diff-changed", *have_next_diffs)
         self.current_path = cursor_path
 
         paths = self._get_selected_paths(pane)
@@ -918,7 +1001,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             treeview.handler_unblock(outid)
 
     def on_treeview_focus_in_event(self, tree, event):
-        self.treeview_focussed = tree
+        self.focus_pane = tree
         pane = self.treeview.index(tree)
         self.actiongroup.get_action("DirCopyLeft").set_sensitive(pane > 0)
         self.actiongroup.get_action("DirCopyRight").set_sensitive(pane+1 < self.num_panes)
@@ -930,12 +1013,16 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.actiongroup.get_action("DirCopyRight").set_sensitive(False)
         self.actiongroup.get_action("DirDelete").set_sensitive(False)
 
-        #
-        # Toolbar handlers
-        #
-
     def on_button_diff_clicked(self, button):
-        self.launch_comparisons_on_selected()
+        pane = self._get_focused_pane()
+        if pane is None:
+            return
+
+        selected = self._get_selected_paths(pane)
+        for row in selected:
+            row_paths = self.model.value_paths(self.model.get_iter(row))
+            paths = [p for p in row_paths if os.path.exists(p)]
+            self.emit("create-diff", paths)
 
     def on_button_copy_left_clicked(self, button):
         self.copy_selected(-1)
@@ -1008,7 +1095,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             is_present = [ os.path.exists( f ) for f in curfiles ]
             all_present = 0 not in is_present
             if all_present:
-                if _files_same(curfiles, regexes) in (Same, SameFiltered):
+                if _files_same(curfiles, regexes, self.prefs) in (Same, SameFiltered):
                     state = tree.STATE_NORMAL
                 else:
                     state = tree.STATE_MODIFIED
@@ -1024,19 +1111,23 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         files = self.model.value_paths(it)
         regexes = [f.filter for f in self.text_filters if f.active]
 
-        def mtime(f):
+        def stat(f):
             try:
-                return os.stat(f).st_mtime
+                return os.stat(f)
             except OSError:
-                return 0
+                return None
+        stats = [stat(f) for f in files[:self.num_panes]]
+        sizes = [s.st_size if s else 0 for s in stats]
+        perms = [s.st_mode if s else 0 for s in stats]
+
         # find the newest file, checking also that they differ
-        mod_times = [ mtime(f) for f in files[:self.num_panes] ]
+        mod_times = [s.st_mtime if s else 0 for s in stats]
         newest_index = mod_times.index( max(mod_times) )
         if mod_times.count( max(mod_times) ) == len(mod_times):
             newest_index = -1 # all same
         all_present = 0 not in mod_times
         if all_present:
-            all_same = _files_same(files, regexes)
+            all_same = _files_same(files, regexes, self.prefs)
             all_present_same = all_same
         else:
             lof = []
@@ -1044,7 +1135,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 if mod_times[j]:
                     lof.append( files[j] )
             all_same = Different
-            all_present_same = _files_same(lof, regexes)
+            all_present_same = _files_same(lof, regexes, self.prefs)
         different = 1
         one_isdir = [None for i in range(self.model.ntree)]
         for j in range(self.model.ntree):
@@ -1069,6 +1160,42 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                     self.model.column_index(COL_EMBLEM, j),
                     j == newest_index and "emblem-meld-newer-file" or None)
                 one_isdir[j] = isdir
+
+                # A DateCellRenderer would be nicer, but potentially very slow
+                TIME = self.model.column_index(COL_TIME, j)
+                mod_datetime = datetime.datetime.fromtimestamp(mod_times[j])
+                time_str = mod_datetime.strftime("%a %d %b %Y %H:%M:%S")
+                self.model.set_value(it, TIME, time_str)
+
+                def natural_size(bytes):
+                    suffixes = (
+                            'B', 'kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'
+                            )
+                    size = float(bytes)
+                    unit = 0
+                    while size > 1000 and unit < len(suffixes) - 1:
+                        size /= 1000
+                        unit += 1
+                    format_str = "%.1f %s" if unit > 0 else "%d %s"
+                    return format_str % (size, suffixes[unit])
+
+                # A SizeCellRenderer would be nicer, but potentially very slow
+                SIZE = self.model.column_index(COL_SIZE, j)
+                size_str = natural_size(sizes[j])
+                self.model.set_value(it, SIZE, size_str)
+
+                def format_mode(mode):
+                    perms = []
+                    rwx = ((4, 'r'), (2, 'w'), (1, 'x'))
+                    for group_index in (6, 3, 0):
+                        group = mode >> group_index & 7
+                        perms.extend([p if group & i else '-' for i, p in rwx])
+                    return "".join(perms)
+
+                PERMS = self.model.column_index(COL_PERMS, j)
+                perm_str = format_mode(perms[j])
+                self.model.set_value(it, PERMS, perm_str)
+
         for j in range(self.model.ntree):
             if not mod_times[j]:
                 self.model.set_path_state(it, j, tree.STATE_NONEXIST,
@@ -1125,7 +1252,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 if treeview.row_expanded(rowiter.path):
                     for row in rowiter.iterchildren():
                         recurse_tree_states(row)
-            recurse_tree_states(iter(self.model).next())
+            recurse_tree_states(next(iter(self.model)))
             row_states.append(None)
 
             numlines = float(len(row_states) - 1)
@@ -1153,11 +1280,13 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             toshow =  self.scrolledwindow[:n] + self.fileentry[:n]
             toshow += self.linkmap[:n-1] + self.diffmap[:n]
             toshow += self.vbox[:n] + self.msgarea_mgr[:n]
-            map( lambda x: x.show(), toshow )
+            for widget in toshow:
+                widget.show()
             tohide =  self.scrolledwindow[n:] + self.fileentry[n:]
             tohide += self.linkmap[n-1:] + self.diffmap[n:]
             tohide += self.vbox[n:] + self.msgarea_mgr[n:]
-            map( lambda x: x.hide(), tohide )
+            for widget in tohide:
+                widget.hide()
             if self.num_panes != 0: # not first time through
                 self.num_panes = n
                 self.on_fileentry_activate(None)
@@ -1173,11 +1302,23 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
     def recompute_label(self):
         root = self.model.get_iter_root()
         filenames = self.model.value_paths(root)
-        shortnames = misc.shorten_names(*filenames)
+        if self.custom_labels:
+            label_options = zip(self.custom_labels, filenames)
+            shortnames = [l[0] or l[1] for l in label_options]
+        else:
+            shortnames = misc.shorten_names(*filenames)
         self.label_text = " : ".join(shortnames)
         self.tooltip_text = self.label_text
         self.label_changed()
 
+    def set_labels(self, labels):
+        labels = labels[:self.num_panes]
+        extra = self.num_panes - len(labels)
+        if extra:
+            labels.extend([""] * extra)
+        self.custom_labels = labels
+        self.recompute_label()
+
     def _update_diffmaps(self):
         self.diffmap[0].queue_draw()
         self.diffmap[1].queue_draw()
@@ -1222,8 +1363,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self._update_item_state( model.get_iter(path) )
 
     def next_diff(self, direction):
-        if self.treeview_focussed:
-            pane = self.treeview.index(self.treeview_focussed)
+        if self.focus_pane:
+            pane = self.treeview.index(self.focus_pane)
         else:
             pane = 0
         if direction == gtk.gdk.SCROLL_UP:
