@@ -13,7 +13,8 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
 import codecs
 import copy
@@ -31,20 +32,22 @@ import gobject
 import gtk
 import gtk.keysyms
 
-import diffutil
-from ui import findbar
-from ui import gnomeglade
-import matchers
-import misc
-import meldbuffer
-import melddoc
-import patchdialog
-import paths
-import merge
-import undo
+from . import diffutil
+from . import matchers
+from . import meldbuffer
+from . import melddoc
+from . import merge
+from . import misc
+from . import patchdialog
+from . import paths
+from . import recent
+from . import undo
+from .ui import findbar
+from .ui import gnomeglade
 
-from meldapp import app
-from util.sourceviewer import srcviewer
+from .meldapp import app
+from .util.compat import text_type
+from .util.sourceviewer import srcviewer
 
 
 def init_worker():
@@ -80,7 +83,7 @@ class CachedSequenceMatcher(object):
         except KeyError:
             def inline_cb(opcodes):
                 self.cache[(text1, textn)] = [opcodes, time.time()]
-                cb(opcodes)
+                gobject.idle_add(lambda: cb(opcodes))
             process_pool.apply_async(matcher_worker, (text1, textn),
                                      callback=inline_cb)
 
@@ -143,7 +146,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                  gtk.keysyms.Control_R : MASK_CTRL}
 
     # Identifiers for MsgArea messages
-    (MSG_SAME,) = range(1)
+    (MSG_SAME,) = list(range(1))
 
     __gsignals__ = {
         'next-conflict-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (bool, bool)),
@@ -155,7 +158,18 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         """
         melddoc.MeldDoc.__init__(self, prefs)
         gnomeglade.Component.__init__(self, paths.ui_dir("filediff.ui"), "filediff")
-        self.map_widgets_into_lists(["textview", "fileentry", "diffmap", "scrolledwindow", "linkmap", "statusimage", "msgarea_mgr", "vbox"])
+        self.map_widgets_into_lists(["textview", "fileentry", "diffmap",
+                                     "scrolledwindow", "linkmap",
+                                     "statusimage", "msgarea_mgr", "vbox",
+                                     "selector_hbox", "readonlytoggle"])
+
+        # This SizeGroup isn't actually necessary for FileDiff; it's for
+        # handling non-homogenous selectors in FileComp. It's also fragile.
+        column_sizes = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        column_sizes.set_ignore_hidden(True)
+        for widget in self.selector_hbox:
+            column_sizes.add_widget(widget)
+
         self.warned_bad_comparison = False
         # Some sourceviews bind their own undo mechanism, which we replace
         gtk.binding_entry_remove(srcviewer.GtkTextView, gtk.keysyms.z,
@@ -174,7 +188,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.load_font()
         self.deleted_lines_pending = -1
         self.textview_overwrite = 0
-        self.textview_focussed = None
+        self.focus_pane = None
         self.textview_overwrite_handlers = [ t.connect("toggle-overwrite", self.on_textview_toggle_overwrite) for t in self.textview ]
         self.textbuffer = [v.get_buffer() for v in self.textview]
         self.buffer_texts = [meldbuffer.BufferLines(b) for b in self.textbuffer]
@@ -201,15 +215,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         for buf in self.textbuffer:
             buf.create_tag("inline")
 
-        # We need to keep track of gtk.TextIter validity, but this isn't
-        # exposed anywhere. Instead, we keep our own counter for changes
-        # across all of our buffers.
-        self._buffer_changed_stamp = 0
-        def buffer_change(buf):
-            self._buffer_changed_stamp += 1
-        for buf in self.textbuffer:
-            buf.connect("changed", buffer_change)
-
         actions = (
             ("MakePatch", None, _("Format as patch..."), None, _("Create a patch using differences between files"), self.make_patch),
             ("PrevConflict", None, _("Previous conflict"), "<Ctrl>I", _("Go to the previous conflict"), lambda x: self.on_next_conflict(gtk.gdk.SCROLL_UP)),
@@ -311,8 +316,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
     def on_container_switch_in_event(self, ui):
         melddoc.MeldDoc.on_container_switch_in_event(self, ui)
         # FIXME: If no focussed textview, action sensitivity will be unset
-        if self.textview_focussed:
-            self.scheduler.add_task(self.textview_focussed.grab_focus)
 
     def on_text_filters_changed(self, app):
         relevant_change = self.create_text_filters()
@@ -338,8 +341,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 buf.disconnect(h)
 
     def _connect_buffer_handlers(self):
-        for textview in self.textview:
-            textview.set_editable(1)
+        for textview, buf in zip(self.textview, self.textbuffer):
+            textview.set_editable(buf.data.editable)
         for buf in self.textbuffer:
             id0 = buf.connect("insert-text", self.on_text_insert_text)
             id1 = buf.connect("delete-range", self.on_text_delete_range)
@@ -375,19 +378,19 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.status_info_labels[1].set_text(line_column)
 
         if line != self.cursor.line or force:
-            chunk, prev, next = self.linediffer.locate_chunk(pane, line)
+            chunk, prev, next_ = self.linediffer.locate_chunk(pane, line)
             if chunk != self.cursor.chunk or force:
                 self.cursor.chunk = chunk
                 self.emit("current-diff-changed")
-            if prev != self.cursor.prev or next != self.cursor.next or force:
+            if prev != self.cursor.prev or next_ != self.cursor.next or force:
                 self.emit("next-diff-changed", prev is not None,
-                          next is not None)
+                          next_ is not None)
 
             prev_conflict, next_conflict = None, None
             for conflict in self.linediffer.conflicts:
                 if prev is not None and conflict <= prev:
                     prev_conflict = conflict
-                if next is not None and conflict >= next:
+                if next_ is not None and conflict >= next_:
                     next_conflict = conflict
                     break
             if prev_conflict != self.cursor.prev_conflict or \
@@ -395,20 +398,26 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 self.emit("next-conflict-changed", prev_conflict is not None,
                           next_conflict is not None)
 
-            self.cursor.prev, self.cursor.next = prev, next
+            self.cursor.prev, self.cursor.next = prev, next_
             self.cursor.prev_conflict = prev_conflict
             self.cursor.next_conflict = next_conflict
         self.cursor.line, self.cursor.offset = line, offset
 
     def on_current_diff_changed(self, widget, *args):
-        pane = self.cursor.pane
-        chunk_id = self.cursor.chunk
-        push_left, push_right, pull_left, pull_right, delete, \
-            copy_left, copy_right = (True,) * 7
+        pane = self._get_focused_pane()
+        if pane != -1:
+            # While this *should* be redundant, it's possible for focus pane
+            # and cursor pane to be different in several situations.
+            pane = self.cursor.pane
+            chunk_id = self.cursor.chunk
+
         if pane == -1 or chunk_id is None:
             push_left, push_right, pull_left, pull_right, delete, \
                 copy_left, copy_right = (False,) * 7
         else:
+            push_left, push_right, pull_left, pull_right, delete, \
+                copy_left, copy_right = (True,) * 7
+
             # Push and Delete are active if the current pane has something to
             # act on, and the target pane exists and is editable. Pull is
             # sensitive if the source pane has something to get, and the
@@ -561,7 +570,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         # This hack is required when pane0's prev/next chunk doesn't exist
         # (i.e., is Same) between pane0 and pane1.
         prev_chunk0, prev_chunk1, next_chunk0, next_chunk1 = (None,) * 4
-        _, prev, next = self.linediffer.locate_chunk(pane0, line)
+        _, prev, next_ = self.linediffer.locate_chunk(pane0, line)
         if prev is not None:
             while prev >= 0:
                 prev_chunk0 = self.linediffer.get_chunk(prev, pane0, pane1)
@@ -572,15 +581,15 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                     break
                 prev -= 1
 
-        if next is not None:
-            while next < self.linediffer.diff_count():
-                next_chunk0 = self.linediffer.get_chunk(next, pane0, pane1)
-                next_chunk1 = self.linediffer.get_chunk(next, pane1, pane0)
+        if next_ is not None:
+            while next_ < self.linediffer.diff_count():
+                next_chunk0 = self.linediffer.get_chunk(next_, pane0, pane1)
+                next_chunk1 = self.linediffer.get_chunk(next_, pane1, pane0)
                 if None not in (next_chunk0, next_chunk1):
                     end0 = next_chunk0[1]
                     end1 = next_chunk1[1]
                     break
-                next += 1
+                next_ += 1
 
         return "Same", start0, end0, start1, end1
 
@@ -645,11 +654,14 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.textview[new_pane].scroll_to_mark(new_buf.get_insert(), 0.1)
 
     def on_textview_focus_in_event(self, view, event):
-        self.textview_focussed = view
+        self.focus_pane = view
         self.findbar.textview = view
         self.on_cursor_position_changed(view.get_buffer(), None, True)
         self._set_merge_action_sensitivity()
 
+    def on_textview_focus_out_event(self, view, event):
+        self._set_merge_action_sensitivity()
+
     def _after_text_modified(self, buffer, startline, sizechange):
         if self.num_panes > 1:
             pane = self.textbuffer.index(buffer)
@@ -701,7 +713,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         fontdesc = pango.FontDescription(self.prefs.get_current_font())
         context = self.textview0.get_pango_context()
         metrics = context.get_metrics( fontdesc, context.get_language() )
-        self.pixels_per_line = (metrics.get_ascent() + metrics.get_descent()) / 1024
+        line_height_points = metrics.get_ascent() + metrics.get_descent()
+        self.pixels_per_line = line_height_points // 1024
         self.pango_char_width = metrics.get_approximate_char_width()
         tabs = pango.TabArray(10, 0)
         tab_size = self.prefs.tab_size
@@ -767,33 +780,40 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         if self.keymask & ~x != self.keymask:
             self.keymask &= ~x
 
-    def on_delete_event(self, appquit=0):
+    def check_save_modified(self, label=None):
         response = gtk.RESPONSE_OK
         modified = [b.data.modified for b in self.textbuffer]
         if True in modified:
-            dialog = gnomeglade.Component(paths.ui_dir("filediff.ui"), "closedialog")
+            ui_path = paths.ui_dir("filediff.ui")
+            dialog = gnomeglade.Component(ui_path, "check_save_dialog")
             dialog.widget.set_transient_for(self.widget.get_toplevel())
+            if label:
+                dialog.widget.props.text = label
+            # FIXME: Should be packed into dialog.widget.get_message_area(),
+            # but this is unbound on currently required PyGTK.
             buttons = []
             for i in range(self.num_panes):
-                b = gtk.CheckButton(self.textbuffer[i].data.label)
-                b.set_use_underline(False)
-                buttons.append(b)
-                dialog.box.pack_start(b, 1, 1)
-                if not modified[i]:
-                    b.set_sensitive(0)
-                else:
-                    b.set_active(1)
-            dialog.box.show_all()
+                button = gtk.CheckButton(self.textbuffer[i].data.label)
+                button.set_use_underline(False)
+                button.set_sensitive(modified[i])
+                button.set_active(modified[i])
+                dialog.extra_vbox.pack_start(button, expand=True, fill=True)
+                buttons.append(button)
+            dialog.extra_vbox.show_all()
             response = dialog.widget.run()
-            try_save = [ b.get_active() for b in buttons]
+            try_save = [b.get_active() for b in buttons]
             dialog.widget.destroy()
-            if response==gtk.RESPONSE_OK:
+            if response == gtk.RESPONSE_OK:
                 for i in range(self.num_panes):
                     if try_save[i]:
                         if not self.save_file(i):
                             return gtk.RESPONSE_CANCEL
             elif response == gtk.RESPONSE_DELETE_EVENT:
                 response = gtk.RESPONSE_CANCEL
+        return response
+
+    def on_delete_event(self, appquit=0):
+        response = self.check_save_modified()
         if response == gtk.RESPONSE_OK:
             for h in self.app_handlers:
                 app.disconnect(h)
@@ -818,13 +838,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.undosequence.end_group()
 
     def on_text_insert_text(self, buf, it, text, textlen):
-        text = unicode(text, 'utf8')
+        text = text_type(text, 'utf8')
         self.undosequence.add_action(
             meldbuffer.BufferInsertionAction(buf, it.get_offset(), text))
         buf.create_mark("insertion-start", it, True)
 
     def on_text_delete_range(self, buf, it0, it1):
-        text = unicode(buf.get_text(it0, it1, False), 'utf8')
+        text = text_type(buf.get_text(it0, it1, False), 'utf8')
         assert self.deleted_lines_pending == -1
         self.deleted_lines_pending = it1.get_line() - it0.get_line()
         self.undosequence.add_action(
@@ -850,24 +870,24 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             buf = self.textbuffer[pane]
             sel = buf.get_selection_bounds()
             if sel:
-                return unicode(buf.get_text(sel[0], sel[1], False), 'utf8')
+                return text_type(buf.get_text(sel[0], sel[1], False), 'utf8')
         return None
 
     def on_find_activate(self, *args):
         selected_text = self.get_selected_text()
-        self.findbar.start_find(self.textview_focussed, selected_text)
+        self.findbar.start_find(self.focus_pane, selected_text)
         self.keymask = 0
 
     def on_replace_activate(self, *args):
         selected_text = self.get_selected_text()
-        self.findbar.start_replace(self.textview_focussed, selected_text)
+        self.findbar.start_replace(self.focus_pane, selected_text)
         self.keymask = 0
 
     def on_find_next_activate(self, *args):
-        self.findbar.start_find_next(self.textview_focussed)
+        self.findbar.start_find_next(self.focus_pane)
 
     def on_find_previous_activate(self, *args):
-        self.findbar.start_find_previous(self.textview_focussed)
+        self.findbar.start_find_previous(self.focus_pane)
 
     def on_filediff__key_press_event(self, entry, event):
         if event.keyval == gtk.keysyms.Escape:
@@ -933,8 +953,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                     stock = gtk.STOCK_SAVE
                 else:
                     stock = gtk.STOCK_SAVE_AS
-            elif not self.textbuffer[i].data.writable:
-                stock = gtk.STOCK_NO
             if stock:
                 self.statusimage[i].show()
                 self.statusimage[i].set_from_stock(stock, gtk.ICON_SIZE_MENU)
@@ -960,7 +978,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.recompute_label()
         self.textview[len(files) >= 2].grab_focus()
         self._connect_buffer_handlers()
-        self.scheduler.add_task( self._set_files_internal(files).next )
+        self.scheduler.add_task(self._set_files_internal(files))
+
+    def get_comparison(self):
+        files = [b.data.filename for b in self.textbuffer[:self.num_panes]]
+        return recent.TYPE_FILE, files
 
     def _load_files(self, files, textbuffers):
         self.undosequence.clear()
@@ -1052,13 +1074,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         yield _("[%s] Computing differences") % self.label_text
         texts = self.buffer_filtered[:self.num_panes]
         step = self.linediffer.set_sequences_iter(texts)
-        while step.next() is None:
+        while next(step) is None:
             yield 1
 
-        chunk, prev, next = self.linediffer.locate_chunk(1, 0)
+        chunk, prev, next_ = self.linediffer.locate_chunk(1, 0)
         self.cursor.next = chunk
         if self.cursor.next is None:
-            self.cursor.next = next
+            self.cursor.next = next_
         for buf in self.textbuffer:
             buf.place_cursor(buf.get_start_iter())
         self.scheduler.add_task(lambda: self.next_diff(gtk.gdk.SCROLL_DOWN), True)
@@ -1096,7 +1118,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self._disconnect_buffer_handlers()
         self.linediffer.clear()
         self.queue_draw()
-        self.scheduler.add_task(self._diff_files().next)
+        self.scheduler.add_task(self._diff_files())
 
     def _set_merge_action_sensitivity(self):
         pane = self._get_focused_pane()
@@ -1152,19 +1174,28 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 # We don't use self.buffer_texts here, as removing line
                 # breaks messes with inline highlighting in CRLF cases
                 text1 = bufs[0].get_text(starts[0], ends[0], False)
-                text1 = unicode(text1, 'utf8')
+                text1 = text_type(text1, 'utf8')
                 textn = bufs[1].get_text(starts[1], ends[1], False)
-                textn = unicode(textn, 'utf8')
+                textn = text_type(textn, 'utf8')
 
                 # For very long sequences, bail rather than trying a very slow comparison
-                inline_limit = 8000 # arbitrary constant
+                inline_limit = 8000
                 if len(text1) + len(textn) > inline_limit:
                     for i in range(2):
                         bufs[i].apply_tag(tags[i], starts[i], ends[i])
                     continue
 
-                def apply_highlight(bufs, tags, starts, change_stamp, matches):
-                    if change_stamp != self._buffer_changed_stamp:
+                def apply_highlight(bufs, tags, starts, ends, texts, matches):
+                    starts = [bufs[0].get_iter_at_mark(starts[0]),
+                              bufs[1].get_iter_at_mark(starts[1])]
+                    ends = [bufs[0].get_iter_at_mark(ends[0]),
+                            bufs[1].get_iter_at_mark(ends[1])]
+                    text1 = bufs[0].get_text(starts[0], ends[0], False)
+                    text1 = text_type(text1, 'utf8')
+                    textn = bufs[1].get_text(starts[1], ends[1], False)
+                    textn = text_type(textn, 'utf8')
+
+                    if texts != (text1, textn):
                         return
 
                     # Remove equal matches of size greater than 3; highlight
@@ -1184,10 +1215,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                     bufs[0].remove_tag(tags[0], starts[0], ends[0])
                     bufs[1].remove_tag(tags[1], starts[1], ends[1])
 
-                match_cb = functools.partial(apply_highlight,
-                                             bufs, tags, starts,
-                                             self._buffer_changed_stamp)
-                matches = self._cached_match.match(text1, textn, match_cb)
+                starts = [bufs[0].create_mark(None, starts[0], True),
+                          bufs[1].create_mark(None, starts[1], True)]
+                ends = [bufs[0].create_mark(None, ends[0], True),
+                        bufs[1].create_mark(None, ends[1], True)]
+                match_cb = functools.partial(apply_highlight, bufs, tags,
+                                             starts, ends, (text1, textn))
+                self._cached_match.match(text1, textn, match_cb)
 
         self._cached_match.clean(self.linediffer.diff_count())
 
@@ -1365,16 +1399,17 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
     def save_file(self, pane, saveas=0):
         buf = self.textbuffer[pane]
         bufdata = buf.data
-        if saveas or not bufdata.filename:
-            filename = self._get_filename_for_saving( _("Choose a name for buffer %i.") % (pane+1) )
+        if saveas or not bufdata.filename or not bufdata.writable:
+            prompt = _("Choose a name for buffer %i.") % (pane + 1)
+            filename = self._get_filename_for_saving(prompt)
             if filename:
                 bufdata.filename = bufdata.label = os.path.abspath(filename)
-                self.fileentry[pane].set_filename( bufdata.filename)
+                self.fileentry[pane].set_filename(bufdata.filename)
                 self.fileentry[pane].prepend_history(bufdata.filename)
             else:
                 return False
         start, end = buf.get_bounds()
-        text = unicode(buf.get_text(start, end, False), 'utf8')
+        text = text_type(buf.get_text(start, end, False), 'utf8')
         if bufdata.newlines:
             if type(bufdata.newlines) == type(""):
                 if(bufdata.newlines) != '\n':
@@ -1413,14 +1448,26 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         dialog = patchdialog.PatchDialog(self)
         dialog.run()
 
-    def set_buffer_writable(self, buf, yesno):
-        buf.data.writable = yesno
+    def set_buffer_writable(self, buf, writable):
+        buf.data.writable = writable
         self.recompute_label()
+        index = self.textbuffer.index(buf)
+        self.readonlytoggle[index].set_visible(not writable)
+        self.set_buffer_editable(buf, writable)
 
     def set_buffer_modified(self, buf, yesno):
         buf.data.modified = yesno
         self.recompute_label()
 
+    def set_buffer_editable(self, buf, editable):
+        buf.data.editable = editable
+        index = self.textbuffer.index(buf)
+        self.readonlytoggle[index].set_active(not editable)
+        self.textview[index].set_editable(editable)
+        self.on_cursor_position_changed(buf, None, True)
+        for linkmap in self.linkmap:
+            linkmap.queue_draw()
+
     def save(self):
         pane = self._get_focused_pane()
         if pane >= 0:
@@ -1437,10 +1484,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 self.save_file(i)
 
     def on_fileentry_activate(self, entry):
-        if self.on_delete_event() != gtk.RESPONSE_CANCEL:
-            files = [e.get_full_path() for e in self.fileentry[:self.num_panes]]
-            self.set_files(files)
-        return 1
+        if self.check_save_modified() != gtk.RESPONSE_CANCEL:
+            entries = self.fileentry[:self.num_panes]
+            paths = [e.get_full_path() for e in entries]
+            self.set_files(paths)
+        return True
 
     def _get_focused_pane(self):
         for i in range(self.num_panes):
@@ -1448,18 +1496,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 return i
         return -1
 
-        #
-        # refresh and reload
-        #
     def on_reload_activate(self, *extra):
-        modified = [os.path.basename(b.data.label) for b in self.textbuffer if b.data.modified]
-        if len(modified):
-            message = _("Reloading will discard changes in:\n%s\n\nYou cannot undo this operation.") % "\n".join(modified)
-            response = misc.run_dialog( message, parent=self, messagetype=gtk.MESSAGE_WARNING, buttonstype=gtk.BUTTONS_OK_CANCEL)
-            if response != gtk.RESPONSE_OK:
-                return
-        files = [b.data.filename for b in self.textbuffer[:self.num_panes]]
-        self.set_files(files)
+        primary = _("Save changes to documents before reloading?")
+        if self.check_save_modified(primary) != gtk.RESPONSE_CANCEL:
+            files = [b.data.filename for b in self.textbuffer[:self.num_panes]]
+            self.set_files(files)
 
     def on_refresh_activate(self, *extra):
         self.refresh_comparison()
@@ -1485,6 +1526,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.actiongroup.get_action("LockScrolling").set_active(locked)
         self._scroll_lock = not locked
 
+    def on_readonly_button_toggled(self, button):
+        index = self.readonlytoggle.index(button)
+        buf = self.textbuffer[index]
+        self.set_buffer_editable(buf, not button.get_active())
+
         #
         # scrollbars
         #
@@ -1560,12 +1606,16 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             toshow =  self.scrolledwindow[:n] + self.fileentry[:n]
             toshow += self.vbox[:n] + self.msgarea_mgr[:n]
             toshow += self.linkmap[:n-1] + self.diffmap[:n]
-            map( lambda x: x.show(), toshow )
+            toshow += self.selector_hbox[:n]
+            for widget in toshow:
+                widget.show()
 
             tohide =  self.statusimage + self.scrolledwindow[n:] + self.fileentry[n:]
             tohide += self.vbox[n:] + self.msgarea_mgr[n:]
             tohide += self.linkmap[n-1:] + self.diffmap[n:]
-            map( lambda x: x.hide(), tohide )
+            tohide += self.selector_hbox[n:]
+            for widget in tohide:
+                widget.hide()
 
             right_attach = 2 * n
             if self.findbar.widget in self.table:
@@ -1635,7 +1685,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         b0, b1 = self.textbuffer[src], self.textbuffer[dst]
         start = b0.get_iter_at_line_or_eof(chunk[1])
         end = b0.get_iter_at_line_or_eof(chunk[2])
-        t0 = unicode(b0.get_text(start, end, False), 'utf8')
+        t0 = text_type(b0.get_text(start, end, False), 'utf8')
 
         if copy_up:
             if chunk[2] >= b0.get_line_count() and \
@@ -1665,7 +1715,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         src_end = b0.get_iter_at_line_or_eof(chunk[2])
         dst_start = b1.get_iter_at_line_or_eof(chunk[3])
         dst_end = b1.get_iter_at_line_or_eof(chunk[4])
-        t0 = unicode(b0.get_text(src_start, src_end, False), 'utf8')
+        t0 = text_type(b0.get_text(src_start, src_end, False), 'utf8')
         mark0 = b1.create_mark(None, dst_start, True)
         self.on_textbuffer__begin_user_action()
         b1.delete(dst_start, dst_end)
