@@ -13,7 +13,8 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
 
 import os
@@ -24,7 +25,7 @@ import gobject
 import gio
 import gtk
 
-import task
+from . import task
 
 from gettext import gettext as _
 
@@ -61,6 +62,10 @@ class MeldDoc(gobject.GObject):
     def get_info_widgets(self):
         return self.status_info_labels
 
+    def get_comparison(self):
+        """Get the comparison type and path(s) being compared"""
+        pass
+
     def save(self):
         pass
 
@@ -148,6 +153,8 @@ class MeldDoc(gobject.GObject):
         uimanager.insert_action_group(self.actiongroup, -1)
         self.popup_menu = uimanager.get_widget("/Popup")
         uimanager.ensure_update()
+        if hasattr(self, "focus_pane") and self.focus_pane:
+            self.scheduler.add_task(self.focus_pane.grab_focus)
 
     def on_container_switch_out_event(self, uimanager):
         """Called when the container app switches away from this tab.
@@ -157,17 +164,8 @@ class MeldDoc(gobject.GObject):
 
     def on_delete_event(self, appquit=0):
         """Called when the docs container is about to close.
-           A doc normally returns gtk.RESPONSE_OK but may return
-           gtk.RESPONSE_CANCEL which requests the container
-           to not delete it. In the special case when the
-           app is about to quit, gtk.RESPONSE_CLOSE may be returned
-           which instructs the container to quit without any
-           more callbacks.
+
+        A doc normally returns gtk.RESPONSE_OK, but may instead return
+        gtk.RESPONSE_CANCEL to request that the container not delete it.
         """
         return gtk.RESPONSE_OK
-
-    def on_quit_event(self):
-        """Called when the docs container is about to close.
-           There is no way to interrupt the quit event.
-        """
-        pass
