@@ -13,7 +13,8 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
 import os
 from gettext import gettext as _
@@ -22,72 +23,23 @@ import gio
 import gtk
 import gobject
 
-import dirdiff
-import filediff
-import filemerge
-import paths
-from ui import gnomeglade
-import misc
-from ui import notebooklabel
-import preferences
-import task
-import vcview
+from . import dirdiff
+from . import filediff
+from . import filemerge
+from . import melddoc
+from . import misc
+from . import newdifftab
+from . import paths
+from . import preferences
+from . import recent
+from . import task
+from . import vcview
+from .ui import gnomeglade
+from .ui import notebooklabel
 
-from util.sourceviewer import srcviewer
-from meldapp import app
-
-################################################################################
-#
-# NewDocDialog
-#
-################################################################################
-
-class NewDocDialog(gnomeglade.Component):
-    def __init__(self, parentapp):
-        gnomeglade.Component.__init__(self, paths.ui_dir("meldapp.ui"), "newdialog")
-        self.map_widgets_into_lists(["fileentry", "direntry", "vcentry", "three_way_compare"])
-        self.entrylists = self.fileentry, self.direntry, self.vcentry
-        self.widget.set_transient_for(parentapp.widget)
-        self.fileentry[0].set_sensitive(self.three_way_compare[0].get_active())
-        self.direntry[0].set_sensitive(self.three_way_compare[1].get_active())
-        self.diff_methods = (parentapp.append_filediff,
-                             parentapp.append_dirdiff,
-                             parentapp.append_vcview)
-        self.parentapp = parentapp
-        self.widget.show_all()
-
-    def on_entry_activate(self, entry):
-        for el in self.entrylists:
-            if entry in el:
-                i = el.index(entry)
-                if i == len(el) - 1:
-                    self.button_ok.grab_focus()
-                else:
-                    el[i+1].focus_entry()
-
-    def on_three_way_toggled(self, button):
-        page = self.three_way_compare.index(button)
-        self.entrylists[page][0].set_sensitive( button.get_active() )
-        self.entrylists[page][not button.get_active()].focus_entry()
-
-    def on_response(self, dialog, arg):
-        if arg == gtk.RESPONSE_OK:
-            page = self.type_notebook.get_current_page()
-            paths = [e.get_full_path() or "" for e in self.entrylists[page]]
-            if page < 2 and not self.three_way_compare[page].get_active():
-                paths.pop(0)
-            for path in paths:
-                self.entrylists[page][0].prepend_history(path)
-            if page == 2:
-                new_tab = self.diff_methods[page](paths[0])
-            else:
-                new_tab = self.diff_methods[page](paths)
-
-            # We just opened a new comparison, transfer focus to it
-            new_tab_idx = self.parentapp.notebook.page_num(new_tab.widget)
-            self.parentapp.notebook.set_current_page(new_tab_idx)
-
-        self.widget.destroy()
+from .util.compat import string_types
+from .util.sourceviewer import srcviewer
+from .meldapp import app
 
 
 ################################################################################
@@ -108,7 +60,7 @@ class MeldWindow(gnomeglade.Component):
 
         actions = (
             ("FileMenu", None, _("_File")),
-            ("New",     gtk.STOCK_NEW,      _("_New..."), "<control>N", _("Start a new comparison"), self.on_menu_file_new_activate),
+            ("New",     gtk.STOCK_NEW,      _("_New comparison"), "<control>N", _("Start a new comparison"), self.on_menu_file_new_activate),
             ("Save",    gtk.STOCK_SAVE,     None, None, _("Save the current file"), self.on_menu_save_activate),
             ("SaveAs",  gtk.STOCK_SAVE_AS,  None, "<control><shift>S", "Save the current file with a different name", self.on_menu_save_as_activate),
             ("Close",   gtk.STOCK_CLOSE,    None, None, _("Close the current file"), self.on_menu_close_activate),
@@ -160,6 +112,15 @@ class MeldWindow(gnomeglade.Component):
         self.actiongroup.set_translation_domain("meld")
         self.actiongroup.add_actions(actions)
         self.actiongroup.add_toggle_actions(toggleactions)
+
+        recent_action = gtk.RecentAction("Recent",  _("Open Recent"),
+                                         _("Open recent files"), None)
+        recent_action.set_show_private(True)
+        recent_action.set_filter(app.recent_comparisons.recent_filter)
+        recent_action.set_sort_type(gtk.RECENT_SORT_MRU)
+        recent_action.connect("item-activated", self.on_action_recent)
+        self.actiongroup.add_action(recent_action)
+
         self.ui = gtk.UIManager()
         self.ui.insert_action_group(self.actiongroup, 0)
         self.ui.add_ui_from_file(ui_file)
@@ -255,7 +216,7 @@ class MeldWindow(gnomeglade.Component):
 
     def on_idle(self):
         ret = self.scheduler.iteration()
-        if ret and isinstance(ret, basestring):
+        if ret and isinstance(ret, string_types):
             self.statusbar.set_task_status(ret)
 
         pending = self.scheduler.tasks_pending()
@@ -286,6 +247,9 @@ class MeldWindow(gnomeglade.Component):
     def on_delete_event(self, *extra):
         return self.on_menu_quit_activate()
 
+    def has_pages(self):
+        return self.notebook.get_n_pages() > 0
+
     def _update_page_action_sensitivity(self):
         current_page = self.notebook.get_current_page()
         have_prev_tab = current_page > 0
@@ -307,7 +271,8 @@ class MeldWindow(gnomeglade.Component):
         oldidx = notebook.get_current_page()
         if oldidx >= 0:
             olddoc = notebook.get_nth_page(oldidx).get_data("pyobject")
-            olddoc.disconnect(self.diff_handler)
+            if self.diff_handler is not None:
+                olddoc.disconnect(self.diff_handler)
             olddoc.on_container_switch_out_event(self.ui)
             if self.undo_handlers:
                 undoseq = olddoc.undosequence
@@ -328,13 +293,20 @@ class MeldWindow(gnomeglade.Component):
         self.actiongroup.get_action("Undo").set_sensitive(can_undo)
         self.actiongroup.get_action("Redo").set_sensitive(can_redo)
 
-        nbl = self.notebook.get_tab_label( newdoc.widget )
+        nbl = self.notebook.get_tab_label(newdoc.widget)
         self.widget.set_title(nbl.get_label_text() + " - Meld")
-        self.statusbar.set_info_box(newdoc.get_info_widgets())
-        self.diff_handler = newdoc.connect("next-diff-changed",
-                                           self.on_next_diff_changed)
+        try:
+            self.statusbar.set_info_box(newdoc.get_info_widgets())
+        except AttributeError:
+            pass
         newdoc.on_container_switch_in_event(self.ui)
-        self.scheduler.add_task( newdoc.scheduler )
+        if isinstance(newdoc, melddoc.MeldDoc):
+            self.diff_handler = newdoc.connect("next-diff-changed",
+                                               self.on_next_diff_changed)
+        else:
+            self.diff_handler = None
+        if hasattr(newdoc, 'scheduler'):
+            self.scheduler.add_task(newdoc.scheduler)
 
     def after_switch_page(self, notebook, page, which):
         self._update_page_action_sensitivity()
@@ -381,7 +353,7 @@ class MeldWindow(gnomeglade.Component):
     # Toolbar and menu items (file)
     #
     def on_menu_file_new_activate(self, menuitem):
-        NewDocDialog(self)
+        self.append_new_comparison()
 
     def on_menu_save_activate(self, menuitem):
         self.current_doc().save()
@@ -389,6 +361,16 @@ class MeldWindow(gnomeglade.Component):
     def on_menu_save_as_activate(self, menuitem):
         self.current_doc().save_as()
 
+    def on_action_recent(self, action):
+        uri = action.get_current_uri()
+        if not uri:
+            return
+        try:
+            self.append_recent(uri)
+        except (IOError, ValueError):
+            # FIXME: Need error handling, but no sensible display location
+            pass
+
     def on_menu_close_activate(self, *extra):
         i = self.notebook.get_current_page()
         if i >= 0:
@@ -404,8 +386,6 @@ class MeldWindow(gnomeglade.Component):
             response = self.try_remove_page(page, appquit=1)
             if response == gtk.RESPONSE_CANCEL:
                 return gtk.RESPONSE_CANCEL
-        for c in self.notebook.get_children():
-            c.get_data("pyobject").on_quit_event()
         gtk.main_quit()
         return gtk.RESPONSE_CLOSE
 
@@ -566,16 +546,19 @@ class MeldWindow(gnomeglade.Component):
         "See if a page will allow itself to be removed"
         response = page.on_delete_event(appquit)
         if response != gtk.RESPONSE_CANCEL:
-            self.scheduler.remove_scheduler(page.scheduler)
+            if hasattr(page, 'scheduler'):
+                self.scheduler.remove_scheduler(page.scheduler)
             page_num = self.notebook.page_num(page.widget)
             assert page_num >= 0
 
             # If the page we're removing is the current page, we need to
             # disconnect and clear undo handlers, and trigger a switch out
             if self.notebook.get_current_page() == page_num:
-                page.disconnect(self.diff_handler)
-                for handler in self.undo_handlers:
-                    page.undosequence.disconnect(handler)
+                if self.diff_handler is not None:
+                    page.disconnect(self.diff_handler)
+                if self.undo_handlers:
+                    for handler in self.undo_handlers:
+                        page.undosequence.disconnect(handler)
                 self.undo_handlers = tuple()
                 page.on_container_switch_out_event(self.ui)
 
@@ -592,23 +575,41 @@ class MeldWindow(gnomeglade.Component):
                 page.on_file_changed(filename)
 
     def _append_page(self, page, icon):
-        nbl = notebooklabel.NotebookLabel(icon, "", lambda b: self.try_remove_page(page))
-        self.notebook.append_page( page.widget, nbl)
+        nbl = notebooklabel.NotebookLabel(icon, "",
+                                          lambda b: self.try_remove_page(page))
+        self.notebook.append_page(page.widget, nbl)
 
         # Change focus to the newly created page only if the user is on a
-        # DirDiff or VcView page.  This prevents cycling through X pages
-        # when X diffs are initiated.
+        # DirDiff or VcView page, or if it's a new tab page. This prevents
+        # cycling through X pages when X diffs are initiated.
         if isinstance(self.current_doc(), dirdiff.DirDiff) or \
-           isinstance(self.current_doc(), vcview.VcView):
+           isinstance(self.current_doc(), vcview.VcView) or \
+           isinstance(page, newdifftab.NewDiffTab):
             self.notebook.set_current_page(self.notebook.page_num(page.widget))
 
-        self.scheduler.add_scheduler(page.scheduler)
-        page.connect("label-changed", self.on_notebook_label_changed)
-        page.connect("file-changed", self.on_file_changed)
-        page.connect("create-diff", lambda obj,arg: self.append_diff(arg) )
+        if hasattr(page, 'scheduler'):
+            self.scheduler.add_scheduler(page.scheduler)
+        if isinstance(page, melddoc.MeldDoc):
+            page.connect("label-changed", self.on_notebook_label_changed)
+            page.connect("file-changed", self.on_file_changed)
+            page.connect("create-diff", lambda obj, arg: self.append_diff(arg))
+            page.connect("status-changed",
+                         lambda obj, arg: self.statusbar.set_doc_status(arg))
 
-        # Allow reordering of tabs
-        self.notebook.set_tab_reorderable(page.widget, True);
+        self.notebook.set_tab_reorderable(page.widget, True)
+
+    def append_new_comparison(self):
+        doc = newdifftab.NewDiffTab(self)
+        self._append_page(doc, "document-new")
+        self.on_notebook_label_changed(doc, _("New comparison"), None)
+
+        def diff_created_cb(doc, newdoc):
+            self.try_remove_page(doc)
+            idx = self.notebook.page_num(newdoc.widget)
+            self.notebook.set_current_page(idx)
+
+        doc.connect("diff-created", diff_created_cb)
+        return doc
 
     def append_dirdiff(self, dirs, auto_compare=False):
         assert len(dirs) in (1,2,3)
@@ -667,11 +668,26 @@ class MeldWindow(gnomeglade.Component):
         doc = vcview.VcView(app.prefs)
         # FIXME: need a good themed VC icon
         self._append_page(doc, "vc-icon")
+        location = location[0] if isinstance(location, list) else location
         doc.set_location(location)
         if auto_compare:
             doc.on_button_diff_clicked(None)
         return doc
 
+    def append_recent(self, uri):
+        comparison_type, files, flags = app.recent_comparisons.read(uri)
+        if comparison_type == recent.TYPE_MERGE:
+            tab = self.append_filemerge(files)
+        elif comparison_type == recent.TYPE_FOLDER:
+            tab = self.append_dirdiff(files)
+        elif comparison_type == recent.TYPE_VC:
+            # Files should be a single-element iterable
+            tab = self.append_vcview(files[0])
+        else:  # comparison_type == recent.TYPE_FILE:
+            tab = self.append_filediff(files)
+        app.recent_comparisons.add(tab)
+        return tab
+
     def _single_file_open(self, path):
         doc = vcview.VcView(app.prefs)
         def cleanup():
@@ -694,13 +710,18 @@ class MeldWindow(gnomeglade.Component):
 
         elif len(paths) in (2, 3):
             tab = self.append_diff(paths, auto_compare, auto_merge)
+        if tab:
+            app.recent_comparisons.add(tab)
         return tab
 
     def current_doc(self):
         "Get the current doc or a dummy object if there is no current"
         index = self.notebook.get_current_page()
         if index >= 0:
-            return self.notebook.get_nth_page(index).get_data("pyobject")
+            page = self.notebook.get_nth_page(index).get_data("pyobject")
+            if isinstance(page, melddoc.MeldDoc):
+                return page
+
         class DummyDoc(object):
             def __getattr__(self, a): return lambda *x: None
         return DummyDoc()
