@@ -13,29 +13,38 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
 
 from gettext import gettext as _
 
 import gtk
 
-from ui import gnomeglade
-from ui import listwidget
-import meldapp
-import misc
-import paths
-from util import prefs
-import vc
+from . import filters
+from . import misc
+from . import paths
+from . import vc
+from .ui import gnomeglade
+from .ui import listwidget
+from .util import prefs
 
-from util.sourceviewer import srcviewer
+from .util.sourceviewer import srcviewer
+
+
+TIMESTAMP_RESOLUTION_PRESETS = [('1ns (ext4)', 1),
+                                ('100ns (NTFS)', 100),
+                                ('1s (ext2/ext3)', 1000000000),
+                                ('2s (VFAT)', 2000000000)]
 
 
 class FilterList(listwidget.ListWidget):
 
     def __init__(self, prefs, key, filter_type):
         default_entry = [_("label"), False, _("pattern"), True]
-        listwidget.ListWidget.__init__(self, default_entry)
+        listwidget.ListWidget.__init__(self, "EditableList.ui",
+                                       "list_alignment", ["EditableListStore"],
+                                       "EditableList", default_entry)
         self.prefs = prefs
         self.key = key
         self.filter_type = filter_type
@@ -44,7 +53,7 @@ class FilterList(listwidget.ListWidget):
                                                self.valid_icon_celldata)
 
         for filtstring in getattr(self.prefs, self.key).split("\n"):
-            filt = meldapp.FilterEntry.parse(filtstring, filter_type)
+            filt = filters.FilterEntry.parse(filtstring, filter_type)
             if filt is None:
                 continue
             valid = filt.filter is not None
@@ -69,7 +78,7 @@ class FilterList(listwidget.ListWidget):
         self.model[path][1] = not ren.get_active()
 
     def on_pattern_edited(self, ren, path, text):
-        filt = meldapp.FilterEntry.compile_filter(text, self.filter_type)
+        filt = filters.FilterEntry.compile_filter(text, self.filter_type)
         valid = filt is not None
         self.model[path][2] = text
         self.model[path][3] = valid
@@ -85,6 +94,46 @@ class FilterList(listwidget.ListWidget):
         setattr(self.prefs, self.key, "\n".join(pref))
 
 
+class ColumnList(listwidget.ListWidget):
+
+    available_columns = set((
+        "size",
+        "modification time",
+        "permissions",
+    ))
+
+    def __init__(self, prefs, key):
+        listwidget.ListWidget.__init__(self, "EditableList.ui",
+                               "columns_ta", ["ColumnsListStore"],
+                               "columns_treeview")
+        self.prefs = prefs
+        self.key = key
+
+        prefs_columns = []
+        for column in getattr(self.prefs, self.key):
+            column_name, visibility = column.rsplit(" ", 1)
+            visibility = bool(int(visibility))
+            prefs_columns.append((column_name, visibility))
+
+        missing = self.available_columns - set([c[0] for c in prefs_columns])
+        prefs_columns.extend([(m, False) for m in missing])
+        for column_name, visibility in prefs_columns:
+            self.model.append([visibility, _(column_name.capitalize())])
+
+        for signal in ('row-changed', 'row-deleted', 'row-inserted',
+                       'rows-reordered'):
+            self.model.connect(signal, self._update_columns)
+
+        self._update_sensitivity()
+
+    def on_cellrenderertoggle_toggled(self, ren, path):
+        self.model[path][0] = not ren.get_active()
+
+    def _update_columns(self, *args):
+        columns = ["%s %d" % (c[1].lower(), int(c[0])) for c in self.model]
+        setattr(self.prefs, self.key, columns)
+
+
 class PreferencesDialog(gnomeglade.Component):
 
     def __init__(self, parent, prefs):
@@ -134,17 +183,38 @@ class PreferencesDialog(gnomeglade.Component):
 
         # file filters
         self.filefilter = FilterList(self.prefs, "filters",
-                                     meldapp.FilterEntry.SHELL)
+                                     filters.FilterEntry.SHELL)
         self.file_filters_tab.pack_start(self.filefilter.widget)
         self.checkbutton_ignore_symlinks.set_active( self.prefs.ignore_symlinks)
 
         # text filters
         self.textfilter = FilterList(self.prefs, "regexes",
-                                     meldapp.FilterEntry.REGEX)
+                                     filters.FilterEntry.REGEX)
         self.text_filters_tab.pack_start(self.textfilter.widget)
         self.checkbutton_ignore_blank_lines.set_active( self.prefs.ignore_blank_lines )
         # encoding
         self.entry_text_codecs.set_text( self.prefs.text_codecs )
+
+        columnlist = ColumnList(self.prefs, "dirdiff_columns")
+        self.column_list_vbox.pack_start(columnlist.widget)
+
+        self.checkbutton_shallow_compare.set_active(
+                self.prefs.dirdiff_shallow_comparison)
+
+        self.combo_timestamp.lock = True
+        model = gtk.ListStore(str, int)
+        active_idx = 0
+        for i, entry in enumerate(TIMESTAMP_RESOLUTION_PRESETS):
+            model.append(entry)
+            if entry[1] == self.prefs.dirdiff_time_resolution_ns:
+                active_idx = i
+        self.combo_timestamp.set_model(model)
+        cell = gtk.CellRendererText()
+        self.combo_timestamp.pack_start(cell, False)
+        self.combo_timestamp.add_attribute(cell, 'text', 0)
+        self.combo_timestamp.set_active(active_idx)
+        self.combo_timestamp.lock = False
+
         self.widget.show()
 
     def on_fontpicker_font_set(self, picker):
@@ -202,6 +272,14 @@ class PreferencesDialog(gnomeglade.Component):
         # Called on "activate" and "focus-out-event"
         self.prefs.text_codecs = entry.props.text
 
+    def on_checkbutton_shallow_compare_toggled(self, check):
+        self.prefs.dirdiff_shallow_comparison = check.get_active()
+
+    def on_combo_timestamp_changed(self, combo):
+        if not combo.lock:
+            resolution = combo.get_model()[combo.get_active_iter()][1]
+            self.prefs.dirdiff_time_resolution_ns = resolution
+
     def on_response(self, dialog, response_id):
         self.widget.destroy()
 
@@ -253,6 +331,14 @@ class MeldPreferences(prefs.Preferences):
                                           ['normal', 'modified', 'new']),
         "vc_status_filters": prefs.Value(prefs.LIST,
                                          ['flatten', 'modified']),
+        # Currently, we're using a quite simple format to store the columns:
+        # each line contains a column name followed by a 1 or a 0
+        # depending on whether the column is visible or not.
+        "dirdiff_columns": prefs.Value(prefs.LIST,
+                                         ["size 1", "modification time 1",
+                                          "permissions 0"]),
+        "dirdiff_shallow_comparison" : prefs.Value(prefs.BOOL, False),
+        "dirdiff_time_resolution_ns" : prefs.Value(prefs.INT, 100),
     }
 
     def __init__(self):
