@@ -12,7 +12,8 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
 import os
 import gobject
@@ -20,7 +21,7 @@ import gtk
 import pango
 
 COL_PATH, COL_STATE, COL_TEXT, COL_ICON, COL_TINT, COL_FG, COL_STYLE, \
-    COL_WEIGHT, COL_STRIKE, COL_END = range(10)
+    COL_WEIGHT, COL_STRIKE, COL_END = list(range(10))
 
 COL_TYPES = (str, str, str, str, str, gtk.gdk.Color, pango.Style,
              pango.Weight, bool)
