@@ -22,11 +22,13 @@
 ### (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 ### THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-import os
 import errno
-import _vc
+import os
 import re
 
+from . import _vc
+
+
 class Vc(_vc.CachedVc):
 
     CMD = "bzr"
@@ -34,6 +36,7 @@ class Vc(_vc.CachedVc):
     NAME = "Bazaar"
     VC_DIR = ".bzr"
     PATCH_INDEX_RE = "^=== modified file '(.*)'.*$"
+    CONFLICT_RE = "conflict in (.*)$"
 
     # We use None here to indicate flags that we don't deal with or care about
     state_1_map = {
@@ -72,11 +75,13 @@ class Vc(_vc.CachedVc):
         return [self.CMD] + self.CMDARGS + ["revert"]
     def resolved_command(self):
         return [self.CMD] + self.CMDARGS + ["resolve"]
+
     def valid_repo(self):
-        if _vc.call([self.CMD, "status"], cwd=self.root):
+        if _vc.call([self.CMD, "info"], cwd=self.root):
             return False
         else:
             return True
+
     def get_working_directory(self, workdir):
         return self.root
 
@@ -93,14 +98,20 @@ class Vc(_vc.CachedVc):
                     raise
         tree_state = {}
         for entry in entries:
-            state_string, name = entry[:3], entry[4:]
+            state_string, name = entry[:3], entry[4:].strip()
             if not re.match(self.valid_status_re, state_string):
                 continue
             # TODO: We don't do anything with exec bit changes.
-            path = os.path.join(branch_root, name.strip())
             state = self.state_1_map.get(state_string[0], None)
             if state is None:
                 state = self.state_2_map.get(state_string[1], _vc.STATE_NORMAL)
+            elif state == _vc.STATE_CONFLICT:
+                real_path_match = re.search(self.CONFLICT_RE, name)
+                if real_path_match is None:
+                    continue
+                name = real_path_match.group(1)
+
+            path = os.path.join(branch_root, name)
             tree_state[path] = state
 
         return tree_state
@@ -111,23 +122,22 @@ class Vc(_vc.CachedVc):
         retfiles = []
         retdirs = []
         bzrfiles = {}
-        for path,state in tree.iteritems():
+        for path, state in tree.items():
             mydir, name = os.path.split(path)
             if path.endswith('/'):
                 mydir, name = os.path.split(mydir)
             if mydir != directory:
                 continue
-            rev, options, tag = "","",""
             if path.endswith('/'):
-                retdirs.append( _vc.Dir(path[:-1], name, state))
+                retdirs.append(_vc.Dir(path[:-1], name, state))
             else:
-                retfiles.append( _vc.File(path, name, state, rev, tag, options) )
+                retfiles.append(_vc.File(path, name, state))
             bzrfiles[name] = 1
         for f,path in files:
             if f not in bzrfiles:
                 #state = ignore_re.match(f) is None and _vc.STATE_NONE or _vc.STATE_IGNORED
                 state = _vc.STATE_NORMAL
-                retfiles.append( _vc.File(path, f, state, "") )
+                retfiles.append(_vc.File(path, f, state))
         for d,path in dirs:
             if d not in bzrfiles:
                 #state = ignore_re.match(f) is None and _vc.STATE_NONE or _vc.STATE_IGNORED
