@@ -28,10 +28,15 @@
 ### (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 ### THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+import errno
 import os
 import re
-import errno
-import _vc
+import shutil
+import subprocess
+import tempfile
+
+from . import _vc
+
 
 class Vc(_vc.CachedVc):
 
@@ -74,6 +79,30 @@ class Vc(_vc.CachedVc):
         return [self.CMD,"rm"]
     def revert_command(self):
         return [self.CMD,"checkout"]
+
+    def get_path_for_repo_file(self, path, commit=None):
+        if commit is None:
+            commit = "HEAD"
+        else:
+            raise NotImplementedError()
+
+        if not path.startswith(self.root + os.path.sep):
+            raise _vc.InvalidVCPath(self, path, "Path not in repository")
+        path = path[len(self.root) + 1:]
+
+        obj = commit + ":" + path
+        process = subprocess.Popen([self.CMD, "cat-file", "blob", obj],
+                                   cwd=self.root, stdout=subprocess.PIPE,
+                                   stderr=subprocess.PIPE)
+        vc_file = process.stdout
+
+        # Error handling here involves doing nothing; in most cases, the only
+        # sane response is to return an empty temp file.
+
+        with tempfile.NamedTemporaryFile(prefix='meld-tmp', delete=False) as f:
+            shutil.copyfileobj(vc_file, f)
+        return f.name
+
     def valid_repo(self):
         # TODO: On Windows, this exit code is wrong under the normal shell; it
         # appears to be correct under the default git bash shell however.
@@ -93,7 +122,7 @@ class Vc(_vc.CachedVc):
             try:
                 # Update the index before getting status, otherwise we could
                 # be reading stale status information
-                _vc.popen(["git", "update-index", "--refresh"],
+                _vc.popen([self.CMD, "update-index", "--refresh"],
                           cwd=self.location)
 
                 # Get the status of files that are different in the "index" vs
@@ -178,19 +207,19 @@ class Vc(_vc.CachedVc):
 
         retfiles = []
         retdirs = []
-        for name,path in files:
+        for name, path in files:
             state = tree.get(path, _vc.STATE_NORMAL)
             meta = self._tree_meta_cache.get(path, "")
             retfiles.append(_vc.File(path, name, state, options=meta))
-        for name,path in dirs:
+        for name, path in dirs:
             # git does not operate on dirs, just files
-            retdirs.append( _vc.Dir(path, name, _vc.STATE_NORMAL))
-        for path, state in tree.iteritems():
+            retdirs.append(_vc.Dir(path, name, _vc.STATE_NORMAL))
+        for path, state in tree.items():
             # removed files are not in the filesystem, so must be added here
-            if state is _vc.STATE_REMOVED:
+            if state in (_vc.STATE_REMOVED, _vc.STATE_MISSING):
                 folder, name = os.path.split(path)
                 if folder == directory:
-                    retfiles.append( _vc.File(path, name, state) )
+                    retfiles.append(_vc.File(path, name, state))
         return retdirs, retfiles
 
     def clean_patch(self, patch):
