@@ -1,4 +1,5 @@
 ### Copyright (C) 2002-2005 Stephen Kennedy <stevek@gnome.org>
+### Copyright (C) 2011-2012 Kai Willadsen <kai.willadsen@gmail.com>
 
 ### Redistribution and use in source and binary forms, with or without
 ### modification, are permitted provided that the following conditions
@@ -21,12 +22,15 @@
 ### (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 ### THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+import errno
 import os
 import re
-import errno
-import _vc
+import shutil
+import tempfile
 import xml.etree.ElementTree as ElementTree
 
+from . import _vc
+
 
 class Vc(_vc.CachedVc):
 
@@ -64,11 +68,51 @@ class Vc(_vc.CachedVc):
         return [self.CMD,"revert"]
     def resolved_command(self):
         return [self.CMD,"resolved"]
+
+    def get_path_for_repo_file(self, path, commit=None):
+        if commit is not None:
+            raise NotImplementedError()
+
+        if not path.startswith(self.root + os.path.sep):
+            raise _vc.InvalidVCPath(self, path, "Path not in repository")
+        path = path[len(self.root) + 1:]
+
+        base, fname = os.path.split(path)
+        svn_path = os.path.join(base, ".svn", "text-base", fname + ".svn-base")
+
+        with tempfile.NamedTemporaryFile(prefix='meld-tmp', delete=False) as f:
+            try:
+                with open(svn_path, 'r') as vc_file:
+                    shutil.copyfileobj(vc_file, f)
+            except IOError as err:
+                if err.errno != errno.ENOENT:
+                    raise
+                # If the repository path doesn't exist, we either have an
+                # invalid path (difficult to check) or a new file. Either way,
+                # we just return an empty file
+
+        return f.name
+
+    def _repo_version_support(self, version):
+        return version < 12
+
     def valid_repo(self):
         if _vc.call([self.CMD, "info"], cwd=self.root):
             return False
+
+        # Check for repository version, trusting format file then entries file
+        format_path = os.path.join(self.root, self.VC_DIR, "format")
+        entries_path = os.path.join(self.root, self.VC_DIR, "entries")
+        if os.path.exists(format_path):
+            version_file = format_path
+        elif os.path.exists(entries_path):
+            version_file = entries_path
         else:
-            return True
+            return False
+
+        with open(version_file) as f:
+            content = f.readline().strip()
+        return self._repo_version_support(int(content))
 
     def switch_to_external_diff(self):
         self.external_diff = "diff"
@@ -127,7 +171,6 @@ class Vc(_vc.CachedVc):
             path = os.path.join(directory, name)
 
             isdir = os.path.isdir(path)
-            options = ""
             if isdir:
                 if os.path.exists(path):
                     state = _vc.STATE_NORMAL
@@ -138,6 +181,6 @@ class Vc(_vc.CachedVc):
                     retdirs.append( _vc.Dir(path,name,state) )
             else:
                 state = self.state_map.get(svn_state, _vc.STATE_NONE)
-                retfiles.append( _vc.File(path, name, state, rev, "", options) )
+                retfiles.append(_vc.File(path, name, state, rev))
 
         return retdirs, retfiles
