@@ -13,23 +13,29 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
+from __future__ import print_function
+
+import atexit
 import tempfile
 import shutil
 import os
+import sys
 from gettext import gettext as _
 
 import gtk
 import pango
 
-import tree
-import misc
-from ui import gnomeglade
-import melddoc
-import paths
-import ui.emblemcellrenderer
-import vc
+from . import melddoc
+from . import misc
+from . import paths
+from . import recent
+from . import tree
+from . import vc
+from .ui import emblemcellrenderer
+from .ui import gnomeglade
 
 ################################################################################
 #
@@ -44,6 +50,34 @@ def _commonprefix(files):
         workdir = os.path.dirname(files[0]) or "."
     return workdir
 
+
+def cleanup_temp():
+    temp_location = tempfile.gettempdir()
+    # The strings below will probably end up as debug log, and are deliberately
+    # not marked for translation.
+    for f in _temp_files:
+        try:
+            assert os.path.exists(f) and os.path.isabs(f) and \
+                   os.path.dirname(f) == temp_location
+            os.remove(f)
+        except:
+            except_str = "{0[0]}: \"{0[1]}\"".format(sys.exc_info())
+            print("File \"{0}\" not removed due to".format(f), except_str,
+                  file=sys.stderr)
+    for f in _temp_dirs:
+        try:
+            assert os.path.exists(f) and os.path.isabs(f) and \
+                   os.path.dirname(f) == temp_location
+            shutil.rmtree(f, ignore_errors=1)
+        except:
+            except_str = "{0[0]}: \"{0[1]}\"".format(sys.exc_info())
+            print("Directory \"{0}\" not removed due to".format(f), except_str,
+                  file=sys.stderr)
+
+_temp_dirs, _temp_files = [], []
+atexit.register(cleanup_temp)
+
+
 ################################################################################
 #
 # CommitDialog
@@ -78,7 +112,8 @@ class CommitDialog(gnomeglade.Component):
         buf = self.textview.get_buffer()
         buf.set_text( gentry.child.get_text() )
 
-COL_LOCATION, COL_STATUS, COL_REVISION, COL_TAG, COL_OPTIONS, COL_END = range(tree.COL_END, tree.COL_END+6)
+COL_LOCATION, COL_STATUS, COL_REVISION, COL_TAG, COL_OPTIONS, COL_END = \
+    list(range(tree.COL_END, tree.COL_END+6))
 
 class VcTreeStore(tree.DiffTreeStore):
     def __init__(self):
@@ -153,16 +188,17 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                        "VcShowIgnored", "VcResolved"):
             button = self.actiongroup.get_action(action)
             button.props.icon_name = button.props.stock_id
-        self.tempdirs = []
         self.model = VcTreeStore()
         self.widget.connect("style-set", self.model.on_style_set)
         self.treeview.set_model(self.model)
-        self.treeview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+        selection = self.treeview.get_selection()
+        selection.set_mode(gtk.SELECTION_MULTIPLE)
+        selection.connect("changed", self.on_treeview_selection_changed)
         self.treeview.set_headers_visible(1)
         self.treeview.set_search_equal_func(self.treeview_search_cb)
         self.current_path, self.prev_path, self.next_path = None, None, None
         column = gtk.TreeViewColumn( _("Name") )
-        renicon = ui.emblemcellrenderer.EmblemCellRenderer()
+        renicon = emblemcellrenderer.EmblemCellRenderer()
         rentext = gtk.CellRendererText()
         column.pack_start(renicon, expand=0)
         column.pack_start(rentext, expand=1)
@@ -217,6 +253,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         if not self.prefs.vc_console_visible:
             self.on_console_view_toggle(self.console_hide_box)
         self.vc = None
+        self.valid_vc_actions = tuple()
         # VC ComboBox
         self.combobox_vcs = gtk.ComboBox()
         self.combobox_vcs.lock = True
@@ -237,13 +274,16 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
     def update_actions_sensitivity(self):
         """Disable actions that use not implemented VC plugin methods
         """
+        valid_vc_actions = ["VcDeleteLocally"]
         for action_name, (meth_name, args) in self.action_vc_cmds_map.items():
             action = self.actiongroup.get_action(action_name)
             try:
                 getattr(self.vc, meth_name)(*args)
                 action.props.sensitive = True
+                valid_vc_actions.append(action_name)
             except NotImplementedError:
                 action.props.sensitive = False
+        self.valid_vc_actions = tuple(valid_vc_actions)
 
     def choose_vc(self, vcs):
         """Display VC plugin(s) that can handle the location"""
@@ -313,9 +353,13 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         # If the user is just diffing a file (ie not a directory), there's no
         # need to scan the rest of the repository
         if os.path.isdir(self.vc.location):
-            self.scheduler.add_task(self._search_recursively_iter(self.model.get_iter_root()).next)
+            root = self.model.get_iter_root()
+            self.scheduler.add_task(self._search_recursively_iter(root))
             self.scheduler.add_task(self.on_treeview_cursor_changed)
 
+    def get_comparison(self):
+        return recent.TYPE_VC, [self.location]
+
     def recompute_label(self):
         self.label_text = os.path.basename(self.location)
         # TRANSLATORS: This is the location of the directory the user is diffing
@@ -348,9 +392,8 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 it = self.model.get_iter_root()
                 root = name
             yield _("[%s] Scanning %s") % (self.label_text, root[prefixlen:])
-            #import time; time.sleep(1.0)
-            
-            entries = filter(showable, self.vc.listdir(root))
+
+            entries = [f for f in self.vc.listdir(root) if showable(f)]
             differences = 0
             for e in entries:
                 differences |= (e.state != tree.STATE_NORMAL)
@@ -374,14 +417,8 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         path = fileentry.get_full_path()
         self.set_location(path)
 
-    def on_quit_event(self):
-        self.scheduler.remove_all_tasks()
-        for f in self.tempdirs:
-            if os.path.exists(f):
-                shutil.rmtree(f, ignore_errors=1)
-
     def on_delete_event(self, appquit=0):
-        self.on_quit_event()
+        self.scheduler.remove_all_tasks()
         return gtk.RESPONSE_OK
 
     def on_row_activated(self, treeview, path, tvc):
@@ -402,11 +439,10 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             retry_diff = False
 
             yield _("[%s] Fetching differences") % self.label_text
-            difffunc = self._command_iter(self.vc.diff_command(),
-                                          path_list, 0).next
+            diffiter = self._command_iter(self.vc.diff_command(), path_list, 0)
             diff = None
             while type(diff) != type(()):
-                diff = difffunc()
+                diff = next(diffiter)
                 yield 1
             prefix, patch = diff[0], diff[1]
 
@@ -427,8 +463,15 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                     self.emit("create-diff", [path])
 
     def run_diff(self, path_list):
-        for path in path_list:
-            self.scheduler.add_task(self.run_diff_iter([path]).next, atfront=1)
+        try:
+            for path in path_list:
+                comp_path = self.vc.get_path_for_repo_file(path)
+                os.chmod(comp_path, 0o444)
+                _temp_files.append(comp_path)
+                self.emit("create-diff", [comp_path, path])
+        except NotImplementedError:
+            for path in path_list:
+                self.scheduler.add_task(self.run_diff_iter([path]), atfront=1)
 
     def on_treeview_popup_menu(self, treeview):
         time = gtk.get_current_event_time()
@@ -469,13 +512,16 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         self.prefs.vc_status_filters = active_filters
         self.refresh()
 
+    def on_treeview_selection_changed(self, selection):
+        model, rows = selection.get_selected_rows()
+        have_selection = bool(rows)
+        for action in self.valid_vc_actions:
+            self.actiongroup.get_action(action).set_sensitive(have_selection)
+
     def _get_selected_files(self):
-        sel = []
-        def gather(model, path, it):
-            sel.append( model.value_path(it,0) )
-        s = self.treeview.get_selection()
-        s.selected_foreach(gather)
-        # remove empty entries and remove trailing slashes
+        model, rows = self.treeview.get_selection().get_selected_rows()
+        sel = [self.model.value_path(self.model.get_iter(r), 0) for r in rows]
+        # Remove empty entries and trailing slashes
         return [x[-1] != "/" and x or x[:-1] for x in sel if x is not None]
 
     def _command_iter(self, command, files, refresh):
@@ -483,7 +529,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
            command was executed in and the output of the command.
         """
         msg = misc.shelljoin(command)
-        yield "[%s] %s" % (self.label_text, msg.replace("\n", u"\u21b2") )
+        yield "[%s] %s" % (self.label_text, msg.replace("\n", "\t"))
         def relpath(pbase, p):
             kill = 0
             if len(pbase) and p.startswith(pbase):
@@ -496,10 +542,11 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         files = [ relpath(workdir, f) for f in files ]
         r = None
         self.consolestream.write( misc.shelljoin(command+files) + " (in %s)\n" % workdir)
-        readfunc = misc.read_pipe_iter(command + files, self.consolestream, workdir=workdir).next
+        readiter = misc.read_pipe_iter(command + files, self.consolestream,
+                                       workdir=workdir)
         try:
             while r is None:
-                r = readfunc()
+                r = next(readiter)
                 self.consolestream.write(r)
                 yield 1
         except IOError as e:
@@ -512,30 +559,31 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
     def _command(self, command, files, refresh=1):
         """Run 'command' on 'files'.
         """
-        self.scheduler.add_task( self._command_iter(command, files, refresh).next )
-        
+        self.scheduler.add_task(self._command_iter(command, files, refresh))
+
     def _command_on_selected(self, command, refresh=1):
         files = self._get_selected_files()
         if len(files):
             self._command(command, files, refresh)
-        else:
-            misc.run_dialog( _("Select some files first."), parent=self, messagetype=gtk.MESSAGE_INFO)
 
     def on_button_update_clicked(self, obj):
-        self._command_on_selected( self.vc.update_command() )
+        self._command_on_selected(self.vc.update_command())
+
     def on_button_commit_clicked(self, obj):
-        dialog = CommitDialog( self )
-        dialog.run()
+        CommitDialog(self).run()
 
     def on_button_add_clicked(self, obj):
         self._command_on_selected(self.vc.add_command())
 
     def on_button_remove_clicked(self, obj):
         self._command_on_selected(self.vc.remove_command())
+
     def on_button_resolved_clicked(self, obj):
         self._command_on_selected(self.vc.resolved_command())
+
     def on_button_revert_clicked(self, obj):
         self._command_on_selected(self.vc.revert_command())
+
     def on_button_delete_clicked(self, obj):
         files = self._get_selected_files()
         for name in files:
@@ -575,7 +623,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             return False
 
         tmpdir = tempfile.mkdtemp("-meld")
-        self.tempdirs.append(tmpdir)
+        _temp_dirs.append(tmpdir)
 
         diffs = []
         for fname in self.vc.get_patch_files(patch):
@@ -599,7 +647,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
 
         if result == 0:
             for d in diffs:
-                os.chmod(d[0], 0444)
+                os.chmod(d[0], 0o444)
                 self.emit("create-diff", d)
             return True
         elif not silent:
@@ -638,7 +686,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 self.model.set_value(newiter, self.model.column_index( tree.COL_PATH, 0), where)
                 self.model.set_path_state(newiter, 0, tree.STATE_NORMAL, True)
                 self.model.remove(it)
-                self.scheduler.add_task( self._search_recursively_iter(newiter).next )
+                self.scheduler.add_task(self._search_recursively_iter(newiter))
         else: # XXX fixme
             self.refresh()
 
@@ -712,6 +760,12 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         cursor_path, cursor_col = self.treeview.get_cursor()
         if not cursor_path:
             self.emit("next-diff-changed", False, False)
+            self.current_path = cursor_path
+            return
+
+        # If invoked directly rather than through a callback, we always check
+        if not args:
+            skip = False
         else:
             try:
                 old_cursor = self.model.get_iter(self.current_path)
@@ -719,17 +773,26 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 # An invalid path gives ValueError; None gives a TypeError
                 skip = False
             else:
+                # We can skip recalculation if the new cursor is between
+                # the previous/next bounds, and we weren't on a changed row
                 state = self.model.get_state(old_cursor, 0)
-                # We can skip recalculation if the new cursor is between the
-                # previous/next bounds, and we weren't on a changed row
-                skip = state in (tree.STATE_NORMAL, tree.STATE_EMPTY) and \
-                       self.prev_path < cursor_path < self.next_path
+                if state not in (tree.STATE_NORMAL, tree.STATE_EMPTY):
+                    skip = False
+                else:
+                    if self.prev_path is None and self.next_path is None:
+                        skip = True
+                    elif self.prev_path is None:
+                        skip = cursor_path < self.next_path
+                    elif self.next_path is None:
+                        skip = self.prev_path < cursor_path
+                    else:
+                        skip = self.prev_path < cursor_path < self.next_path
 
-            if not skip:
-                prev, next = self.model._find_next_prev_diff(cursor_path)
-                self.prev_path, self.next_path = prev, next
-                have_next_diffs = (prev is not None, next is not None)
-                self.emit("next-diff-changed", *have_next_diffs)
+        if not skip:
+            prev, next = self.model._find_next_prev_diff(cursor_path)
+            self.prev_path, self.next_path = prev, next
+            have_next_diffs = (prev is not None, next is not None)
+            self.emit("next-diff-changed", *have_next_diffs)
         self.current_path = cursor_path
 
     def next_diff(self, direction):
